# date:2021.3.13

# A part of NonVisual Desktop Access (NVDA)
# Copyright (C) 2009-2020 NV Access Limited, Aleksey Sadovoy, James Teh, Joseph Lee, Tuukka Ojala,
# Bram Duvigneau
# This file is covered by the GNU General Public License.
# See the file COPYING for more details.

import appModuleHandler,speech
import calendar
import collections
import time
import api,os,gui
import ui,wx


# A named tuple for holding the elapsed and total playing times from Foobar2000's status bar
statusBarTimes = collections.namedtuple('StatusBarTimes', ['elapsed', 'total'])

isDelay=0
title=''
art=''
dictLyrics={}
dic_l={}
#lyricsPath=os.path.join(os.environ['APPDATA'],'foobar2000\lyrics\')
lyricsPath=os.path.join(os.environ['PROGRAMFILES'],'foobar2000\lyrics')

lyricsModeConfig=None
lyricsPathConfig=None
isSpeakLyrics=1



class browseToLyricsPath(gui.contextHelp.ContextHelpMixin,wx.Dialog):
	def __init__(self, parent):
		super().__init__(parent, title="浏览到歌词目录")
		self._parent=parent
		sHelper = gui.guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)
		bHelper = sHelper.addDialogDismissButtons(gui.guiHelper.ButtonHelper(wx.HORIZONTAL), separated=True)
		continueButton = bHelper.addButton(self, label=_("浏览&b"), id=wx.ID_OK)
		continueButton.SetDefault()
		continueButton.Bind(wx.EVT_BUTTON, self.onCreatePortable)
		bHelper.addButton(self, id=wx.ID_CANCEL)
		self.Bind(wx.EVT_BUTTON, self.onCancel, id=wx.ID_CANCEL)

	def onCreatePortable(self, evt):
		startPath=os.path.join(os.environ['PROGRAMFILES'],'foobar2000\lyrics')
		with wx.DirDialog(self._parent,'选择歌词目录', defaultPath=startPath) as d:
			if d.ShowModal() == wx.ID_OK:
				global lyricsPath,lyricsPathConfig
				lyricsPath=d.Path+'\\\\'
				with open(lyricsPathConfig,"w",encoding='UTF-8') as f:
					f.write(lyricsPath)
				self.Destroy()

	def onCancel(self, evt):
		self.Destroy()

def getLyricsFileFromTitle_1():
	global lyricsPath,title
	try:
		tp='%s.lrc' % title[:-14]
		tempPath=os.path.join(lyricsPath,tp)
		return tempPath
	except:
		return ''

def getLyricsFileFromTitle_2():
	global art,lyricsPath,title
	try:
		l=title[:-14].split(' - ')
		tp=''
		ll=l[1].split('] ')
		art=ll[0][1:]
		tp='%s - %s.lrc' % (l[0],ll[1])
	except:
		return ''
	tempPath=os.path.join(lyricsPath,tp)
	return tempPath

def getLyricsFileFromFocus():
	global lyricsPath,title,isStageSpeakLyrics
	o=api.getFocusObject()
	try:
		l=o.name.split(' - ')
		tp=''
		ll=l[1][(len(art)+2):-5]
		if l[0]=='?':
			tp='%s.lrc' % ll
		else:
			tp='%s - %s.lrc' % (l[0],ll)
	except:
		return ''
	tempPath=os.path.join(lyricsPath,tp)
	return tempPath

def getDictionary(path):
			global dictLyrics,dic_l
#		try:
			f=open(path,'r',encoding='UTF-8')
			t=f.read()
			f.close()
			l=t.split('\n')
			l=l[l.index(''):]
			for x in l:
				ll=x.split(']')
				for y in ll:
					if ll[-1]:
						dictLyrics[y[2:-3]]=ll[-1]
						dic_l[y[2:-3]]=y[-2:]
			if len(dictLyrics)==0:
				l=t.split(']')
#				l=l[l.index(''):]
#				l=''.join(l).split(']')
				for x in l:
					ll=x.split('[')
					if not speech.isBlank(ll[0]):
						dictLyrics[ll[1][1:-3]]=ll[0]
						dic_l[ll[1][1:-3]]=ll[1][-2:]

#						dictLyrics[y[2:-3]]=y[-2:]
#						dic_l[y[2:]]=ll[-1]
#		except:
#			pass

def getLyricsDictionary():
	lyricsFile=getLyricsFileFromTitle_1()
	if not os.path.isfile(lyricsFile):
		lyricsFile=getLyricsFileFromTitle_2()
		if not os.path.exists(lyricsFile):
			lyricsFile=getLyricsFileFromFocus()
			if not os.path.exists(lyricsFile):
				return False
	getDictionary(lyricsFile)
	return True

def getParsingFormat(interval):
	"""Attempts to find a suitable parsing format string for a HH:MM:SS, MM:SS or SS -style time interval."""
	timeParts = len(interval.split(":"))
	if timeParts == 1:
		return "%S"
	elif timeParts == 2:
		return "%M:%S"
	elif timeParts == 3:
		return "%H:%M:%S"
	else:
		return None

def getOutputFormat(seconds):
	"""Returns a format string for the given number of seconds with the least leading zeros."""
	if seconds < 60:
		return "%S"
	elif seconds < 3600:
		return "%M:%S"
	else:
		return "%H:%M:%S"

def parseIntervalToTimestamp(interval):
	"""Parses a HH:MM:SS, MM:SS or SS -style interval to a timestamp."""
	format = getParsingFormat(interval)
	return calendar.timegm(time.strptime(interval.strip(), format))

class AppModule(appModuleHandler.AppModule):

	_statusBar = None

	previousElapsed=''
	def event_nameChange(self, obj, nextHandler):
		global isSpeakLyrics,title,dictLyrics,dic_l,isDelay
		try:
			if isSpeakLyrics==1:
				tempTitle=api.getForegroundObject().name
				if title != tempTitle:
					title=tempTitle
					dictLyrics={}
					dic_l={}
				if len(dictLyrics)==0:
					getLyricsDictionary()
				self.elapsed=obj.name.split("|")[4].split("/")[0].split(' ')[1]
				if self.previousElapsed != self.elapsed:
					self.previousElapsed=self.elapsed
					ui.message(dictLyrics[self.elapsed])
		except:
			pass
		nextHandler()

	def script_setLyricsPath(self,gesture):
		global lyricsPath,lyricsPathConfig
		bl=browseToLyricsPath(gui.mainFrame)
		bl.Raise()
		bl.Maximize()
		bl.Show()
	script_setLyricsPath.__doc__=_('浏览保存歌词的目录')
	script_setLyricsPath.category=_('foobar2000')

	def script_checkLyrics(self,gesture):
		global isSpeakLyrics,title

		isSpeakLyrics=(isSpeakLyrics+1)%2
		if isSpeakLyrics==0:
			ui.message('停止自动读歌词')
		if isSpeakLyrics==1:
			ui.message('自动读歌词')
			title=api.getForegroundObject().name
			if not getLyricsDictionary():
				ui.message('未匹配到歌词')

		with open(lyricsModeConfig,'w',encoding='utf-8') as f:
			f.write(str(isSpeakLyrics))
	script_checkLyrics.__doc__=_('设置是否自动读出歌词')
	script_checkLyrics.category=_('foobar2000')


	def event_gainFocus(self, obj, nextHandler):
		global lyricsModeConfig,isSpeakLyrics,lyricsPathConfig,lyricsPath
		if not lyricsModeConfig:
			lyricsModeConfig=os.path.join(os.path.dirname(__file__), r"lyricsModeConfig")
			try:
				with open(lyricsModeConfig,'r',encoding='utf-8') as f:
					isSpeakLyrics=int(f.read())
			except:
				isSpeakLyrics=1
		if not lyricsPathConfig:
			lyricsPathConfig=os.path.join(os.path.dirname(__file__), r"lyricsPathConfig")
			if not os.path.isfile(lyricsPathConfig):
				lyricsPath=os.path.join(os.environ['PROGRAMFILES'],'foobar2000\lyrics\\')
				if not os.path.exists(lyricsPath):
					lyricsPath=os.path.join(os.environ['APPDATA'],'foobar2000\lyrics\\')
			else:
				with open(lyricsPathConfig,'r',encoding='utf-8') as f:
					lyricsPath=f.read()

		if not self._statusBar:
			self._statusBar = api.getStatusBar()
		nextHandler()

	def getElapsedAndTotal(self):
		empty = statusBarTimes(None, None)
		if not self._statusBar:
			return empty
		statusBarContents = self._statusBar.firstChild.name
		try:
			playingTimes = statusBarContents.split("|")[4].split("/")
		except IndexError:
			return empty
		elapsed = playingTimes[0]
		if len(playingTimes) > 1:
			total = playingTimes[1]
		else:
			total = None
		return statusBarTimes(elapsed, total)

	def getElapsedAndTotalIfPlaying(self):
		elapsedAndTotalTime = self.getElapsedAndTotal()
		if elapsedAndTotalTime.elapsed is None and elapsedAndTotalTime.total is None:
			# Translators: Reported when no track is playing in Foobar 2000.
			ui.message(_("No track playing"))
		return elapsedAndTotalTime

	def script_reportRemainingTime(self,gesture):
		elapsedTime, totalTime = self.getElapsedAndTotalIfPlaying()
		if elapsedTime is None or totalTime is None:
			# Translators: Reported if the remaining time can not be calculated in Foobar2000
			msg = _("Unable to determine remaining time")
		else:
			parsedElapsedTime = parseIntervalToTimestamp(elapsedTime)
			parsedTotalTime = parseIntervalToTimestamp(totalTime)
			remainingTime = parsedTotalTime - parsedElapsedTime
			msg = time.strftime(getOutputFormat(remainingTime), time.gmtime(remainingTime))
		ui.message(msg)
	# Translators: The description of an NVDA command for reading the remaining time of the currently playing track in Foobar 2000.
	script_reportRemainingTime.__doc__ = _("Reports the remaining time of the currently playing track, if any")
	script_reportRemainingTime.category=_('foobar2000')

	def script_reportElapsedTime(self,gesture):
		elapsedTime = self.getElapsedAndTotalIfPlaying()[0]
		if elapsedTime is not None:
			ui.message(elapsedTime)
	# Translators: The description of an NVDA command for reading the elapsed time of the currently playing track in Foobar 2000.
	script_reportElapsedTime.__doc__ = _("Reports the elapsed time of the currently playing track, if any")
	script_reportElapsedTime.category=_('foobar2000')

	def script_reportTotalTime(self,gesture):
		totalTime = self.getElapsedAndTotalIfPlaying()[1]
		if totalTime is not None:
			ui.message(totalTime)
		else:
			# Translators: Reported if the total time is not available in Foobar2000
			ui.message(_("Total time not available"))
	# Translators: The description of an NVDA command for reading the length of the currently playing track in Foobar 2000.
	script_reportTotalTime.__doc__ = _("Reports the length of the currently playing track, if any")
	script_reportTotalTime.category=_('foobar2000')

	__gestures = {
		"kb:control+shift+r": "reportRemainingTime",
		"kb:control+shift+e": "reportElapsedTime",
		"kb:control+shift+t": "reportTotalTime",
		"kb:control+shift+l": "checkLyrics",
		"kb:control+shift+o": "setLyricsPath",
	}